#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <WiFiUdp.h>
#include <TimeLib.h>
#include <Timezone.h>
#include <Ticker.h>
#include <ArduinoOTA.h>

Ticker timer;

const char* ssid = "Network ID";
const char* password = "Password";

const char* OTApassword = "Password";

const int ledPin1 = D1; // Pin für LED 1 (Grün)
const int ledPin2 = D2; // Pin für LED 2 (Rot)
const int ledPin3 = D3; // Pin für LED 3 (Blau)

const int mymin = 60;
int restzeit = -1;
int timerDuration = -1;

bool ledState1 = LOW;
bool ledState2 = LOW;
bool ledState3 = LOW;
bool ledState4 = LOW;
bool ledState5 = LOW;
bool ledState6 = LOW;
bool ledState7 = LOW;

WiFiUDP udp;
const char* ntpServer = "europe.pool.ntp.org";
const long utcOffsetInSeconds = 0; // Offset für Mitteleuropäische Zeit (MEZ) in Sekunden (1 Stunde)
TimeChangeRule CEST = {"CEST", Last, Sun, Mar, 2, 120}; // Sommerzeitregel: Letzter Sonntag im März um 2 Uhr
TimeChangeRule CET = {"CET ", Last, Sun, Oct, 3, 60};   // Winterzeitregel: Letzter Sonntag im Oktober um 3 Uhr
Timezone myTimeZone(CEST, CET);

const int NTP_PACKET_SIZE = 48;
byte packetBuffer[NTP_PACKET_SIZE];

ESP8266WebServer server(80);

void setup() {
  Serial.begin(115200);
  ArduinoOTA.begin();
   ArduinoOTA.setPassword(OTApassword);

  pinMode(ledPin1, OUTPUT);
  pinMode(ledPin2, OUTPUT);
  pinMode(ledPin3, OUTPUT);

  timer.attach_ms(1000, timer_isr);

  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.print("Verbunden mit ");
  Serial.println(ssid);
  Serial.print("IP-Adresse: ");
  Serial.println(WiFi.localIP());

  udp.begin(2390); // UDP-Verbindung für NTP-Abfrage

  setSyncProvider(getNtpTime); // Zeitfunktion für die Synchronisierung mit NTP-Server

  server.on("/", handleRoot);
  server.on("/led1", handleLED1);
  server.on("/led2", handleLED2);
  server.on("/led3", handleLED3);
  server.on("/led4", handleLED4);
  server.on("/led5", handleLED5);
  server.on("/led6", handleLED6);
  server.on("/led7", handleLED7);
  server.on("/timer", handleTimerPage);
  server.on("/set-timer", handleSetTimer);

  server.begin();
  Serial.println("HTTP-Server gestartet");
}

void loop() {
  server.handleClient();
  ArduinoOTA.handle();
  

  // Aktuelle Zeit anzeigen
  time_t localTime = myTimeZone.toLocal(now());

  // LED 1 (Grün) entsprechend der festgelegten Zeiten steuern
  controlLEDByTime(ledPin1, 18, 30, 20, 15, ledState1); // Einschalten um 18:30 und Ausschalten um 20:15

  // LED 2 (Rot) entsprechend der festgelegten Zeiten steuern
  // Kein spezifischer Zeitplan für Rot in diesem Beispiel

  // LED 3 (Blau) entsprechend der festgelegten Zeiten steuern
  // Kein spezifischer Zeitplan für Blau in diesem Beispiel

  // LED 4 (Gelb) steuern (kombiniert LED 1 und LED 2)
  controlLEDByTime(ledPin1, 6, 10, 8, 0, ledState4);
  controlLEDByTime(ledPin2, 6, 10, 8, 0, ledState4);

  // LED 5 (Violett) steuern (kombiniert LED 3 und LED 2)
  controlLEDByTime(ledPin3, 12, 0, 13, 30, ledState5);
  controlLEDByTime(ledPin2, 12, 0, 13, 30, ledState5);

  // LED 6 (Light Blue) steuern (kombiniert LED 1 und LED 3)
  controlLEDByTime(ledPin1, 20, 16, 22, 0, ledState6);
  controlLEDByTime(ledPin3, 20, 16, 22, 0, ledState6);

  // LED 7 (Weiß) steuern (kombiniert LED 1, LED 2 und LED 3)
  // Kein spezifischer Zeitplan für Weiß in diesem Beispiel
}

void controlLEDByTime(int pin, int onHour, int onMinute, int offHour, int offMinute, bool &ledState) {
  time_t localTime = myTimeZone.toLocal(now());
  int currentHour = hour(localTime);
  int currentMinute = minute(localTime);

  if (currentHour == onHour && currentMinute == onMinute) {
    ledState = true;
    digitalWrite(pin, HIGH); // LED einschalten
  } else if (currentHour == offHour && currentMinute == offMinute) {
    ledState = false;
    digitalWrite(pin, LOW); // LED ausschalten
  }
}

time_t getNtpTime() {
  IPAddress ntpServerIP;
  while (udp.parsePacket() > 0) ; // Verwerfe alte Pakete
  Serial.println("NTP-Anfrage wird gesendet");
  if (WiFi.hostByName(ntpServer, ntpServerIP)) {
    Serial.println("NTP-Server gefunden");
    sendNTPPacket(ntpServerIP);
    uint32_t beginWait = millis();
    while (millis() - beginWait < 1500) {
      int size = udp.parsePacket();
      if (size >= NTP_PACKET_SIZE) {
        Serial.println("NTP-Antwort empfangen");
        udp.read(packetBuffer, NTP_PACKET_SIZE);
        unsigned long secsSince1900 = (unsigned long)packetBuffer[40] << 24 |
                                      (unsigned long)packetBuffer[41] << 16 |
                                      (unsigned long)packetBuffer[42] << 8 |
                                      (unsigned long)packetBuffer[43];
        return secsSince1900 - 2208988800UL + utcOffsetInSeconds;
      }
    }
    Serial.println("Keine NTP-Antwort erhalten");
  } else {
    Serial.println("Kein NTP-Server gefunden");
  }
  return 0; // Rückgabe bei Fehlschlag
}

void sendNTPPacket(IPAddress& address) {
  memset(packetBuffer, 0, NTP_PACKET_SIZE);
  packetBuffer[0] = 0b11100011; // LI, Version, Mode
  packetBuffer[1] = 0;          // Stratum, or type of clock
  packetBuffer[2] = 6;          // Polling Interval
  packetBuffer[3] = 0xEC;       // Peer Clock Precision
  packetBuffer[12] = 49;
  packetBuffer[13] = 0x4E;
  packetBuffer[14] = 49;
  packetBuffer[15] = 52;

  udp.beginPacket(address, 123);
  udp.write(packetBuffer, NTP_PACKET_SIZE);
  udp.endPacket();
}

void handleRoot() {
  String html = "<html><body>";
  html += "<h1>LED Control</h1>";
  html += "<p>Current Time: ";
  time_t localTime = myTimeZone.toLocal(now());
  html += ctime(&localTime);
  html += "</p>";
  html += "<p><a href='/led1'><button>Green ";
  html += ledState1 ? "OFF" : "ON";
  html += "</button></a></p>";
  html += "<p><a href='/led2'><button>Red ";
  html += ledState2 ? "OFF" : "ON";
  html += "</button></a></p>";
  html += "<p><a href='/led3'><button>Blue ";
  html += ledState3 ? "OFF" : "ON";
  html += "</button></a></p>";  
  html += "<p><a href='/led4'><button>Yellow ";
  html += ledState4 ? "OFF" : "ON";
  html += "</button></a></p>";
  html += "<p><a href='/led5'><button>Violet ";
  html += ledState5 ? "OFF" : "ON";
  html += "</button></a></p>";
  html += "<p><a href='/led6'><button>Light Blue ";
  html += ledState6 ? "OFF" : "ON";
  html += "</button></a></p>";
  html += "<p><a href='/led7'><button>White ";
  html += ledState7 ? "OFF" : "ON";
  html += "</button></a></p>";
  html += "<h1>Time Settings</h1>";
  html += "<p><a href='/timer'><button>Timer</button></a></p>";
  html += "</body></html>";
  server.send(200, "text/html", html);
}

void handleLED1() {
  ledState1 = !ledState1;
  digitalWrite(ledPin1, ledState1 ? HIGH : LOW);
  server.sendHeader("Location", "/");
  server.send(303);
}

void handleLED2() {
  ledState2 = !ledState2;
  digitalWrite(ledPin2, ledState2 ? HIGH : LOW);
  server.sendHeader("Location", "/");
  server.send(303);
}

void handleLED3() {
  ledState3 = !ledState3;
  digitalWrite(ledPin3, ledState3 ? HIGH : LOW);
  server.sendHeader("Location", "/");
  server.send(303);
}

void handleLED4() {
  ledState4 = !ledState4;
  digitalWrite(ledPin1, ledState4 ? HIGH : LOW);
  digitalWrite(ledPin2, ledState4 ? HIGH : LOW);
  server.sendHeader("Location", "/");
  server.send(303);
}

void handleLED5() {
  ledState5 = !ledState5;
  digitalWrite(ledPin2, ledState5 ? HIGH : LOW);
  digitalWrite(ledPin3, ledState5 ? HIGH : LOW);
  server.sendHeader("Location", "/");
  server.send(303);
}

void handleLED6() {
  ledState6 = !ledState6;
  digitalWrite(ledPin1, ledState6 ? HIGH : LOW);
  digitalWrite(ledPin3, ledState6 ? HIGH : LOW);
  server.sendHeader("Location", "/");
  server.send(303);
}

void handleLED7() {
  ledState7 = !ledState7;
  digitalWrite(ledPin1, ledState7 ? HIGH : LOW);
  digitalWrite(ledPin2, ledState7 ? HIGH : LOW);
  digitalWrite(ledPin3, ledState7 ? HIGH : LOW);
  server.sendHeader("Location", "/");
  server.send(303);
}

void handleTimerPage() {
  String html = "<html><body>";
  html += "<h1>Set Timer</h1>";
  html += "<form action='/set-timer' method='get'>";
  html += "<p>Timer duration (minutes): <input type='number' name='duration' /></p>";
  html += "<p><input type='submit' value='Set Timer' /></p>";
  html += "</form>";
  html += "</body></html>";
  server.send(200, "text/html", html);
}

void handleSetTimer() {
  if (server.hasArg("duration")) {
    restzeit = server.arg("duration").toInt() * mymin;
    if (restzeit > 0) {
      timerDuration = restzeit;
    }
  }
  server.sendHeader("Location", "/");
  server.send(303);
}

void timer_isr() {
  if (restzeit >= 0) {
    restzeit--;
      if (restzeit == 0) {
        ledState1 = LOW;
        ledState2 = LOW;
        ledState3 = LOW;
        ledState4 = LOW;
        ledState5 = LOW;
        ledState6 = LOW;
        ledState7 = LOW;
        digitalWrite(ledPin1, LOW);
        digitalWrite(ledPin2, LOW);
        digitalWrite(ledPin3, LOW);
    }
  }
}
